/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.opt.oo;

import java.lang.reflect.Modifier;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.proxy.AbstractProxyClassGenerator;
import org.objectweb.fractal.juliac.ucf.UnifiedClass;
import org.objectweb.fractal.juliac.ucf.UnifiedMethod;
import org.objectweb.fractal.juliac.visit.ClassSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.MethodSourceCodeVisitor;
import org.osoa.sca.annotations.Conversational;
import org.osoa.sca.annotations.EndsConversation;
import org.osoa.sca.annotations.OneWay;
import org.ow2.frascati.tinfi.osoa.CallableReferenceImpl;
import org.ow2.frascati.tinfi.osoa.ConversationImpl;

/**
 * This class generates the source code of SCA callable reference
 * implementations.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class CallableReferenceClassGenerator
extends AbstractProxyClassGenerator {
    
    public CallableReferenceClassGenerator() {
        super();
    }
    
    public CallableReferenceClassGenerator(
        InterfaceType it, MembraneDesc<?> membraneDesc, boolean mergeable,
        Juliac jc ) {
        
        super(it,membraneDesc,mergeable,jc);
    }
    
    /**
     * Return <code>true</code> if some code must be generated by the current
     * source code generator.
     */
    public boolean match() {
        return true;
    }

    /**
     * Return the name of the suffix which is appended to generated class names.
     */
    public String getClassNameSuffix() {
        return "FcCR";
    }
    
    @Override
    public void setMergeable( boolean mergeable ) {
        if(mergeable) {
            String msg = "This generator can not generate mergeable code";
            throw new IllegalArgumentException(msg);
        }
    }

    @Override
    public String[] getClassGenericTypeParameters() {
        return new String[]{"B extends "+it.getFcItfSignature()};
    }

    @Override
    public String getSuperClassName() {
        return CallableReferenceImpl.class.getName()+"<B>";
    }

    @Override
    public String[] getImplementedInterfaceNames() {
        return new String[]{it.getFcItfSignature()};
    }
    
    @Override
    public void generateConstructors( ClassSourceCodeVisitor cv, UnifiedClass proxycl ) {
        super.generateConstructors(cv, proxycl);
        
        // public constructor(Class<B>,B)
        String[] parameters =
            new String[]{ "Class<B> businessInterface", "B service" };
        MethodSourceCodeVisitor mv =
            cv.visitConstructor(Modifier.PUBLIC,null,parameters,null);
        mv.visitBegin();
        mv.visitln("    super(businessInterface,service);");
        mv.visitEnd();
    }
    
    @Override
    public void generateProxyMethodBodyBeforeCode(
        MethodSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {
        super.generateProxyMethodBodyBeforeCode(mv, proxycl, proxym);
    
        Conversational conversationalItf = proxycl.getAnnotation(Conversational.class);
        OneWay oneWayMeth = proxym.getAnnotation(OneWay.class);

        if( conversationalItf != null ) {
            mv.visitln("    "+ConversationImpl.class.getName()+" conv = getConversation();");
            mv.visitln("    pushConversation(conv);");
        }
        mv.visitln("    final B impl = getService();");
        if( oneWayMeth != null ) {
            mv.visitln("    new Thread() {");
            mv.visitln("      public void run() {");
        }
    }
    
    @Override
    public void generateProxyMethodBodyAfterCode(
        MethodSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {
        super.generateProxyMethodBodyAfterCode(mv, proxycl, proxym);
        
        OneWay oneWayMeth = proxym.getAnnotation(OneWay.class);
        EndsConversation end = proxym.getAnnotation(EndsConversation.class);

        if( oneWayMeth != null ) {
            mv.visitln("      }");
            mv.visitln("    }.start();");
        }
        if( end != null ) {
            mv.visitln("      conv.end();");
            mv.visitln("      popConversation();");
        }
    }

    public String getDelegatingInstance( UnifiedClass proxycl, UnifiedMethod proxym ) {
        return "impl";
    }
}
