/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 * Contributor: Philippe Merle
 */

package org.ow2.frascati.tinfi.opt.oo;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.proxy.AbstractInterceptorSourceCodeGenerator;
import org.objectweb.fractal.juliac.ucf.UnifiedClass;
import org.objectweb.fractal.juliac.ucf.UnifiedMethod;
import org.objectweb.fractal.juliac.visit.MethodSourceCodeVisitor;
import org.osoa.sca.annotations.EndsConversation;

/**
 * This class generates the source code of Tinfi interceptors which implement
 * the logic related to the management of component content instances. This
 * interceptor invokes the SCA content controller.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @author Philippe Merle <Philippe.Merle@inria.fr>
 */
public class SCAContentInterceptorSourceCodeGenerator
extends AbstractInterceptorSourceCodeGenerator {
        
    public SCAContentInterceptorSourceCodeGenerator() {
        super();
    }

    public SCAContentInterceptorSourceCodeGenerator(
        InterfaceType it, MembraneDesc<?> membraneDesc, boolean mergeable,
        Juliac jc ) {
        
        super(it,membraneDesc,mergeable,jc);
    }
    
    /**
     * Return the name of the suffix which is appended to generated class names.
     */
    public String getClassNameSuffix() {
        return "SCACC";
    }
        
    /**
     * Return <code>true</code> if some code must be generated by the current
     * source code generator.
     */
    public boolean match() {

        // Skip control interfaces
        String itname = it.getFcItfName();
        if( itname.equals("component") || itname.endsWith("-controller") ) {
            return false;
        }

        // Skip client interfaces
        if( it.isFcClientItf() ) {
            return false;
        }

        // Match all other interfaces (business server)
        return true;
    }

    @Override
    public void setMergeable( boolean mergeable ) {
        if(mergeable) {
            String msg = "This generator can not generate mergeable code";
            throw new IllegalArgumentException(msg);
        }
    }

    @Override
    public void generateProxyMethodBodyBeforeCode(
        MethodSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {
        
        /*
         * Push the request context on the thread local stack and get the
         * content instance which correspond to the current scope policy.
         */
        mv.visit  ("    ");
        mv.visit  (it.getFcItfSignature());
        mv.visit  (" impl = pushFcAndGet(\"");
        mv.visit  (it.getFcItfName());
        mv.visitln("\","+it.getFcItfSignature()+".class,this);");
        
        // TODO the request context created by pushFcAndGet should be given a client proxy (not this which is a server one)
        
        // Begin of the try block
        mv.visitln("    try {");
    }
    
    @Override
    public void generateProxyMethodBodyFinallyCode(
        MethodSourceCodeVisitor mv, UnifiedClass proxycl, UnifiedMethod proxym ) {
        
        // End of the try block
        mv.visitln("    }");
        mv.visitln("    finally {");

        // Release the content instance and pop the request context
        mv.visit("      releaseFcAndPop(impl,");
        if( proxym.getAnnotation(EndsConversation.class) == null ) {
            mv.visit("false");
        }
        else {
            mv.visit("true");
        }
        mv.visitln(");");

        // End of the finally block
        mv.visitln("    }");
    }
}
